import json
import os
import base64
import requests



def encode_image(image_path):
    """Encode the image to base64."""
    try:
        with open(image_path, "rb") as image_file:
            return base64.b64encode(image_file.read()).decode('utf-8')
    except FileNotFoundError:
        print(f"Error: The file {image_path} was not found.")
        return None
    except Exception as e:
        print(f"Error: {e}")
        return None

def answer_formatting(text):
    # Find the indices of the triple single quotes
    start_index = text.find("```json") + len("```json")
    end_index = text.rfind("```")

    # Slice the string to get the content between the quotes
    content_between = text[start_index:end_index]

    # Remove newline characters
    text_formatted = content_between.replace('\n', '')

    return text_formatted

def save_to_json(text, file_name):
    structured_data = {}
    # Attempt to load the modified string as JSON
    try:
        structured_data = json.loads(text)  # Convert string to Python dict/list
        print("Successfully parsed JSON.")
    except json.JSONDecodeError as e:
        print("Error: The response content is still not valid JSON after removing newlines.")
        print(f"JSONDecodeError: {e}")

    # Save the structured data to a JSON file
    try:
        with open(f'./data/json_openAI/{file_name}.json', 'w', encoding='utf-8') as json_file:
            if structured_data != {}:
                json.dump(structured_data, json_file, indent=4,
                          ensure_ascii=False)  # `indent=4` for pretty printing
    except FileNotFoundError:
        print(f"Error: The file ./data/json_openAI/{file_name} was not found.")


def text_extraction_OpenAI(image_path, message, key):
    # assigning API KEY to initialize openai environment
    api_key = key
    # Getting the base64 string
    base64_image = encode_image(image_path)
    file_name_without_extension = os.path.splitext(os.path.basename(image_path))[0]

    headers = {
      "Content-Type": "application/json",
      "Authorization": f"Bearer {api_key}"
    }

    payload = {
      "model": "gpt-4o-mini",
      "messages": [
        {
          "role": "user",
          "content": [
            {
              "type": "text",
              "text": message
            },
            {
              "type": "image_url",
              "image_url": {
                "url": f"data:image/jpeg;base64,{base64_image}"
              }
            }
          ]
        }
      ],
      "max_tokens": 3000
    }

    print("Calling Open API to extract text from images.")

    response = requests.post("https://api.openai.com/v1/chat/completions", headers=headers, json=payload)

    formatted_text = answer_formatting(response.json()["choices"][0]["message"]['content'])
    save_to_json(formatted_text, file_name_without_extension)


    return response.json()["choices"][0]["message"]['content']




import os
from mistralai import Mistral
import base64
import json


def encode_image(image_path):
    """Encode the image to base64."""
    try:
        with open(image_path, "rb") as image_file:
            return base64.b64encode(image_file.read()).decode('utf-8')
    except FileNotFoundError:
        print(f"Error: The file {image_path} was not found.")
        return None
    except Exception as e:
        print(f"Error: {e}")
        return None


def text_formatting(text):
    # Remove newline characters
    content_to_save_no_newlines = text.replace('\n', '')

    return content_to_save_no_newlines


def save_to_json(text, file_name):
    structured_data = {}

    # Attempt to load the modified string as JSON
    try:
        structured_data = json.loads(text)  # Convert string to Python dict/list
        print("Successfully parsed JSON.")
    except json.JSONDecodeError as e:
        print("Error: The response content is still not valid JSON after removing newlines.")
        print(f"JSONDecodeError: {e}")

    # Save the structured data to a JSON file
    try:
        with open(f'./data/json_openAI/{file_name}.json', 'w', encoding='utf-8') as json_file:
            if structured_data != {}:
                json.dump(structured_data, json_file, indent=4,
                          ensure_ascii=False)  # `indent=4` for pretty printing
    except FileNotFoundError:
        print(f"Error: The file ./data/json_openAI/{file_name} was not found.")


def text_extraction_Pixtral(image_path, message, key):
    api_key = key
    model = "pixtral-12b-2409"
    client = Mistral(api_key=api_key)
    base64_image = encode_image(image_path)
    file_name = os.path.splitext(os.path.basename(image_path))[0]

    print("Calling Pixtral API to extract text from images.")


    messages = [
        {
            "role": "user",
            "content": [
                {
                    "type": "text",
                    "text": message
                },
                {
                    "type": "image_url",
                    "image_url": f"data:image/jpeg;base64,{base64_image}"
                }
            ],
            "response_format":
                {
                    "type": "json_object"
                }
        }
    ]

    chat_response = client.chat.complete(
        model=model,
        messages=messages,
        response_format = {
            "type": "json_object",
        }
    )

    # Get the content
    content_to_save = chat_response.choices[0].message.content

    formatted_text = text_formatting(content_to_save)

    save_to_json(formatted_text, file_name)



    return content_to_save



from setuptools import find_packages, setup
import os


setup(
	# Project name:
name='aiextraction',
# Packages to include in the distribution:
packages=find_packages(),
# Project version number:
version='1.0.2',
# Short description of your library:
description='A library for text extraction from images into json files using OpenAI and Pixtral',
# Your name:
author='John Petris',
py_modules=['aiextraction'],
# List project dependencies:
install_requires=[
    'annotated-types >= 0.7.0',
    'anyio >= 4.6.0',
    'certifi >= 2024.8.30',
    'charset-normalizer >= 3.3.2',
    'colorama >= 0.4.6',
    'distro >= 1.9.0',
    'docutils >= 0.21.2',
    'eval_type_backport >= 0.2.0',
    'h11 >= 0.14.0',
    'httpcore >= 1.0.5',
    'httpx >= 0.27.2',
    'idna >= 3.10',
    'importlib_metadata >= 8.5.0',
    'jaraco.classes >= 3.4.0',
    'jaraco.context >= 6.0.1',
    'jaraco.functools >= 4.0.2',
    'jiter >= 0.5.0',
    'jsonpath-python >= 1.0.6',
    'keyring >= 25.4.1',
    'markdown-it-py >= 3.0.0',
    'mdurl >= 0.1.2',
    'mistralai >= 1.1.0',
    'more-itertools >= 10.5.0',
    'mypy-extensions >= 1.0.0',
    'nh3 >= 0.2.18',
    'openai >= 1.47.1',
    'pkginfo >= 1.10.0',
    'pydantic >= 2.9.2',
    'pydantic_core >= 2.23.4',
    'Pygments >= 2.18.0',
    'python-dateutil >= 2.8.2',
    'pywin32-ctypes >= 0.2.3',
    'readme_renderer >= 44.0',
    'requests >= 2.32.3',
    'requests-toolbelt >= 1.0.0',
    'rfc3986 >= 2.0.0',
    'rich >= 13.8.1',
    'setuptools >= 75.1.0',
    'six >= 1.16.0',
    'sniffio >= 1.3.1',
    'tqdm >= 4.66.5',
    'twine >= 5.1.1',
    'typing-inspect >= 0.9.0',
    'typing_extensions >= 4.12.2',
    'urllib3 >= 2.2.3',
    'wheel >= 0.44.0',
    'zipp >= 3.20.2',

],
)